        // Implement a strategy that plays high if a card
        // of the correct suit is available and it will beat
        // all those played so far in the round.
        // Otherwise play low.
        // In order to implement this strategy, a record is
        // kept of the leading card of the current round.
class CautiousPlayer implements CardPlayer {
    public CautiousPlayer(int id){
        // The player has an identity.
        identity = id;
    }

        // Receive a card from the dealer.
        // Keep the hand sorted.
    public void takeCard(SuitedCard c){
        Hand hand = getHand();
        hand.add(c);
        hand.sort();
    }

        // We choose to ignore this, which means we will have
        // an incomplete picture of which cards remain in the
        // other players' hands.
    public void undealtCard(SuitedCard c){
    }

        // Play any card from the hand.
        // We have a free choice of suit.
    public SuitedCard playAnyCard() throws RuntimeException {
        // Choose the highest card from an available suit.
        final String[] suits = SuitedCard.getSuitNames();
        for(int i = 0; i < suits.length; i++){
            SuitedCard card = chooseHighestCardInSuit(suits[i]);
            if(card != null){
                getHand().remove(card);
                return card;
            }
        }
        throw new RuntimeException("No cards available.");
    }

        // We must follow suit, if possible.
    public SuitedCard followSuit(String suit){
        SuitedCard card;
        final SuitedCard[] cards = getHand().getCardsInSuit(suit);
        if(cards.length != 0){
            // See if we have one to beat the leader.
            SuitedCard leader = getLeadingCard();
            card = cards[0];
            if(card.getValue() < leader.getValue()){
                // We don't. Choose the lowest of the suit.
                card = cards[cards.length-1];
            }
        }
        else{
            // We cannot win this round.
            card = chooseLowestCard();
        }
        getHand().remove(card);
        return card;
    }

        // Receive notification of a card played in this round.
        // This could be the card that we have just played.
        // Update our record of the leading card of the round.
    public void viewCard(SuitedCard c){
        SuitedCard leader = getLeadingCard();
        if(leader == null){
            setLeadingCard(c);
        }
        else if(c.getSuit().equals(leader.getSuit()) &&
                c.getValue() > leader.getValue()){
            setLeadingCard(c);
        }
        // else - no better.
    }

        // Notification of who won the round just completed.
        // Clear the leading card, ready for the next round.
    public void roundWinner(int id){
        setLeadingCard(null);
    }

    public int getIdentity(){
        return identity;
    }

    public int numCards(){
        return getHand().numCards();
    }

        // Select a card in the given suit, if any.
        // Return null if none is available.
    private SuitedCard chooseHighestCardInSuit(String suit){
        final SuitedCard[] cards = getHand().getCardsInSuit(suit);
        if(cards.length != 0){
            return cards[0];
        }
        else{
            return null;
        }
    }

        // Select the card with the lowest value.
    private SuitedCard chooseLowestCard(){
        final String[] suits = SuitedCard.getSuitNames();
        SuitedCard lowestSoFar = null;
        for(int i = 0; i < suits.length; i++){
            final SuitedCard[] cards = getHand().getCardsInSuit(suits[i]);
            if(cards.length != 0){
                SuitedCard lowestInSuit = cards[cards.length-1];
                if(lowestSoFar == null){
                    lowestSoFar = lowestInSuit;
                }
                else if(lowestInSuit.getValue() < lowestSoFar.getValue()){
                    lowestSoFar = lowestInSuit;
                }
                // else - not lower.
            }
        }
        return lowestSoFar;
    }

    private Hand getHand(){
        return hand;
    }

    private SuitedCard getLeadingCard(){
        return leadingCard;
    }

    private void setLeadingCard(SuitedCard c){
        leadingCard = c;
    }

    // A collection to hold the player's cards.
    private Hand hand = new Hand();
    // The player's identifying number.
    private final int identity;

    // Keep track of the leading card of the current round.
    private SuitedCard leadingCard = null;
}
